#include <math.h>

#include <QtCore/QFile>
#include <QtGui/QMouseEvent>
#include <QtGui/QMessageBox>
#include <QtGui/QImage>

#include "glwidget.h"
#include "stdinc.h"


GLwidget::GLwidget(QWidget *parent, QGLFormat format)
: QGLWidget(format, parent) 
, m_texId(0)
{
	m_timer = new QTimer(this);
	connect(m_timer, SIGNAL(timeout()), this, SLOT(update()));
	generateTexture();
}       

GLwidget::~GLwidget()
{
}


void GLwidget::initializeGL() 
{ 
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glDisable(GL_LIGHTING);
	glDisable(GL_BLEND);

	// enable texturing here
	// ** TODO **

	glLoadIdentity();     

	m_timer->start(10);
}


void GLwidget::resizeGL(int w, int h) 
{   
	glViewport(0, 0, (GLint)w, (GLint)h);  
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();  

	float r = w/(float)h;
	gluPerspective(60, r, 0.1, 100);
}

void GLwidget::paintGL() 
{   
	glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.5, 0.5, 5, 0.5, 0, 0.5, 0, 1, 0);
	glColor3f(1.0f, 1.0f, 1.0f);

	// ** TODO ** hier mit Hilfe der Texturmatrix die Texturkoordinaten so verschieben
	//            dass sich die Textur auf den Betrachter zu bewegt 
	
	// ** TODO ** hier die Texturkoordinaten eintragen, so dass die Textur
	//            in z-Richtung (= t-Richtung der Textur) fuenf mal wiederholt dargestellt wird
	glBegin(GL_QUADS);
	glVertex3f(0,0,10);
	glVertex3f(0,0,0);
	glVertex3f(1,0,0);
	glVertex3f(1,0,10);
	glEnd();
}

// Qt-slots
void GLwidget::slotTextureFaster()
{
	// ** TODO **
	updateGL();
}

void GLwidget::slotTextureSlower()
{
	// ** TODO **
	updateGL();
}


void GLwidget::slotTexFilterNearest(bool b)
{
	if ( b )
	{
		// ** TODO **
		updateGL();
	}
}

void GLwidget::slotTexFilterLinear(bool b)
{
	if ( b )
	{
		// ** TODO **
		updateGL();
	}
}

void GLwidget::slotTexFilterMipMap(bool b)
{
	if ( b )
	{
		// ** TODO **
		updateGL();
	}
}


// hier wird eine Textur erzeugt und im Array m_pixels abgelegt
//
// Texturgroesse(width,height): (16,16)
// Format:						RGB
// Type:						unsigned byte
void GLwidget::generateTexture()
{
	int i,j;
	for (i = 0 ; i < 16 ; i++) 
	{
		for (j = 0 ; j < 16 ; j++) 
		{
			switch ((i+j) % 4) 
			{
			case 0:
				m_pixels[i][j][0] = 255;
				m_pixels[i][j][1] = 255;
				m_pixels[i][j][2] = 255;
				break;
			case 1:
				m_pixels[i][j][0] = 255;
				m_pixels[i][j][1] = 0;
				m_pixels[i][j][2] = 0;
				break;
			case 2:
				m_pixels[i][j][0] = 0;
				m_pixels[i][j][1] = 0;
				m_pixels[i][j][2] = 255;
				break;
			case 3:
				m_pixels[i][j][0] = 255;
				m_pixels[i][j][1] = 255;
				m_pixels[i][j][2] = 0;
				break;
			}
		}
	}
}
